@@ -13,6 +13,10 @@ class ErrorUtil:
         if self.stopOnError:
             print ("Stopping.")
             sys.exit(1)
+    
+    def logWarning(self, message):
+        if not self.silent:
+            cprint(str("Warning: ") + str(message) + "\n", "YELLOW", file=sys.stderr)
 
     # Option-setting functions
     def setStopOnError(self, stopOnErr):
