@@ -23,10 +23,6 @@ import almost_make.utils.errorUtil as errorUtility
 SPACE_CHARS = re.compile(r'\s+')
 INCLUDE_DIRECTIVE_EXP = re.compile(r"^\s*(include|\.include|-include|sinclude)\s+")
 
-# Some targets need duplicates (e.g. %.o: %.c vs %.o: %.cc)
-DUPLICATE_FORMAT_STR = "[DUPLICATE %d] "
-DUPLICATE_CHECK_EXP  = re.compile(r"^\[DUPLICATE (\d+)\] (.*)$")
-
 # Targets that are used by this parser/should be ignored.
 MAGIC_TARGETS = \
 {
@@ -251,13 +247,10 @@ class MakeUtil:
     # Generate a recipe for [target] and add it to [targets].
     # Returns True if there is now a recipe for [target] in [targets],
     #  False otherwise.
-    def generateRecipeFor(self, target, targets, macros, visitedSet=None):
+    def generateRecipeFor(self, target, targets, macros):
         if target in targets:
             return True
         
-        if not visitedSet:
-            visitedSet = set()
-        
         generatedTarget = False
         potentialNewRules = []
 
@@ -360,22 +353,20 @@ class MakeUtil:
 
     # Get whether [target] needs to be (re)generated. If necessary,
     # creates a rule for [target] and adds it to [targets].
-    def prepareGenerateTarget(self, target, targets, macros, visitedSet=None):
+    def prepareGenerateTarget(self, target, targets, macros, visitingSet=None):
         target = target.strip()
         
-        if visitedSet is None:
-            visitedSet = set()
+        if visitingSet is None:
+            visitingSet = set()
         
-        if not target in visitedSet:
-            visitedSet.add(target)
-        else: # Circular dependency?
+        if target in visitingSet: # Circular dependency?
             self.errorUtil.logWarning("Circular dependency involving %s!!!" % target)
 
             # Just return whether it exists or not.
             return self.findFile(target, macros) == None
         
         if not target in targets:
-            self.generateRecipeFor(target, targets, macros, visitedSet)
+            self.generateRecipeFor(target, targets, macros)
         
         targetPath = self.findFile(target, macros)
         selfExists = targetPath != None
@@ -415,7 +406,11 @@ class MakeUtil:
             if selfMTime < os.path.getmtime(pathToOther):
                 return True
 
-            if self.prepareGenerateTarget(dep, targets, macros, visitedSet):
+            visitingSet.add(target)
+            needGenerateDep = self.prepareGenerateTarget(dep, targets, macros, visitingSet)
+            visitingSet.remove(target)
+
+            if needGenerateDep:
                 return True
         return False
 
